% SplitData(FILENAME, TRIAL_INFO, FS)
%   The SplitData function splits the data at FILENAME into different files
%   according to the information provided in TRIAL_INFO.  The original file 
%   is not changed in any way.  (The function Util_GetFileInfo.m is 
%   required for use.)
%
%   INPUTS:
%       FILENAME - The location of the source data.  A text file is 
%           expected (e.g., 'Data.txt').
%       TRIAL_INFO - An Nx2 matrix, where each row contains the starting
%           time and length (in seconds) of the N trials to be extracted.  
%           A time of 0 s corresponds to sample number 1.
%       FS - The sampling rate (Hz).
%
%   OUTPUTS:
%       none.
%
% Written by Alex Andrews, 2012. Last updated:  February 13th, 2012.

function SplitData(sFilename, nTrialInfo, Fs)

fprintf('\n---------------\n');
fprintf('-- SplitData --\n');
fprintf('---------------\n\n');


% -------------------------------
% -- 1.0 Check input arguments --
% -------------------------------

if nargin~=3
    fprintf(['ERROR:  Wrong number of input arguments ',...
        '(there should be three)\n\n']);
    return
end

if isempty(sFilename)||~ischar(sFilename)
    fprintf('ERROR:  Filename must be a valid string\n\n');
    return
end

if length(sFilename)<5||~strcmp(sFilename(end-2:end),'txt')
    fprintf('ERROR:  Filename must have a .txt extension.\n\n');
    return
end

if ~isnumeric(nTrialInfo)|size(nTrialInfo,2)~=2|any(nTrialInfo<0)
    fprintf(['ERROR:  Trial info must be a Nx2 matrix of positive ',...
        'values\n\n']);
    return
end

if ~isnumeric(Fs)||length(Fs)~=1||Fs<=0
    fprintf('ERROR:  Sampling rate must be a positive scalar\n\n');
    return
end


% ------------------------------------
% -- 2.0 Prepare for data splitting --
% ------------------------------------

% 2.1 Convert trials to sample numbers
nTrialInfo=round(nTrialInfo.*Fs);
nTrialInfo(:,1)=nTrialInfo(:,1)+1;

% 2.2 Warn user if trials overlap
if any(sum(nTrialInfo(1:end-1,:),2)>nTrialInfo(2:end,1))
    fprintf('WARNING:  Some trials overlap...\n\n');
end

% 2.3 Determine file characteristics
[nNumChannels nHeaderLines sDelim sPattern]=Util_GetFileInfo(sFilename);
if isempty(nHeaderLines)|nHeaderLines==-1
    fprintf('ERROR:  File could not be loaded.\n\n')
    return
else
    fprintf('FILE CHARACTERISTICS:\n');
    fprintf('  Channels:  %0.0f\n',nNumChannels);
    fprintf('  Header lines:  %0.0f\n',nHeaderLines);
    fprintf('  Delimiter (string):  %s\n',sDelim);
    fprintf('  Delimiter (ascii):  %0.0f\n\n',double(sDelim)); 
end


% --------------------
% -- 3.0 Split data --
% --------------------

% 3.1 Load data file 
fid=fopen(sFilename); 
if fid==-1
    fprintf('ERROR:  Specified file could not be opened!\n\n');
    return
end
nData=textscan(fid,sPattern,'HeaderLines',nHeaderLines,'Delimiter',sDelim);
fclose(fid);

% 3.2 Get filename without extension or path (used for naming new files)
nLastSlash=find(sFilename=='/'|sFilename=='\',1,'last');
if ~isempty(nLastSlash)
    sOrigFilename=sFilename(nLastSlash+1:end-4);
else
    sOrigFilename=sFilename(1:end-4);
end

% 3.3 Generate filenames and save data by trial
sPattern(sPattern=='f')='e';
fprintf('SAVING NEW DATA FILES\n');
for iFile=1:size(nTrialInfo,1)
    sTrialFilename=sprintf('%s_Trial%0.0f.txt',sOrigFilename,iFile);
    fid=fopen(sTrialFilename,'r');
    if fid~=-1
        fclose(fid);
        fprintf(['  WARNING:  %s aleady exists!  Could not export trial ',...
            '%0.0f.\n'],sTrialFilename,iFile);
    else
        nStart=nTrialInfo(iFile,1);
        nEnd=nTrialInfo(iFile,1)+nTrialInfo(iFile,2)-1;
        if nStart<1||nEnd>size(nData{1},1)
            fprintf(['  WARNING:  Specified range for trial %0.0f ',...
                'not valid (samples %0.0f-%0.0f)\n'],iFile,nStart,nEnd);
        else
            fid=fopen(sTrialFilename,'w');
            fprintf(fid,['The following data corresponds to samples ',...
                '%0.0f-%0.0f in %s.\n'],nStart,nEnd,sFilename);
            for i=nStart:nEnd
                for j=1:length(nData)
                    if isnumeric(nData{j})
                        xOutput=nData{j}(i);
                    else
                        xOutput=nData{j}{i};
                    end
                    fprintf(fid,sPattern(((j-1)*2+1):j*2),xOutput);
                    if j<length(nData),fprintf(fid,'\t');end
                end
                fprintf(fid,'\n');
            end
            fclose(fid);
            fprintf('  %s saved successfully...\n',sTrialFilename);
        end
    end        
end
fprintf('\nDATA SPLITTING COMPLETE!\n\n');